import java.sql.Connection;
import java.sql.SQLException;
import java.sql.*;
import java.util.Vector;

public class DBController implements Constants {
  Connection connection;

  public DBController()throws Exception {
        Class.forName(this.DB_FORNAME);
        connection = DriverManager.getConnection(this.DB_URL+this.DB_NAME,   // filenames
                                           this.DB_USER,                    // username
                                           this.DB_PW);                     // password
  }


    public synchronized void query(String expression) throws SQLException {
        Statement st = null;
        ResultSet rs = null;
        st = connection.createStatement();
        rs = st.executeQuery(expression);
        st.close();
    }



  public String[][] getDegrees(){
    String ret[][]= null;
    Vector vdegrees = new Vector();
    Statement stfarm1;
    ResultSet resfarm1;
    String sql = "SELECT Degree_ID, Degree_name FROM Degrees ORDER BY Degree_ID";
    try{
      stfarm1 = connection.createStatement();
      resfarm1 = stfarm1.executeQuery(sql);
      if(resfarm1.next()){
        do{
          vdegrees.addElement(resfarm1.getInt("Degree_ID")+"#"+resfarm1.getString("Degree_name"));
        }while(resfarm1.next());
      }else{
          System.out.println("NO ELEMENTS FOUND: String[] getDegrees()");
      }
      stfarm1.close();
    }catch(SQLException ex){
        ex.printStackTrace();
    }
    if(!vdegrees.isEmpty()){
      ret = new String[vdegrees.size()][2];
      for(int i=0;i<vdegrees.size();i++){
          String tmp = vdegrees.elementAt(i).toString().trim();
          String tmp_arr[] = tmp.split("#");
          ret[i][0]=tmp_arr[0];
          ret[i][1]=tmp_arr[1];
      }
    }
    return ret;
  }

  public void shutdown() throws SQLException {
        connection.close();   // if there are no other open connection
                        // db writes out to files and shuts down
                        // this happens anyway at garbage collection
                        // when program ends
    }


  public Vector getImagePairs(int ExpID){
    Vector ImagePairs = new Vector();
    Statement st;
    ResultSet res;
    String sql = "SELECT Imagepair_ID, Image_ID1, Image_ID2, Experiment_ID, Same_Individual FROM Imagepair WHERE Experiment_ID= "+ExpID+" ORDER BY Imagepair_ID";
//    System.out.println(sql);
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
      do{
          int p_ID = res.getInt("Imagepair_ID");
          int ID1 = res.getInt("Image_ID1");
          int ID2 = res.getInt("Image_ID2");
          int Exp_ID = res.getInt("Experiment_ID");
          int Same_Individual = res.getInt("Same_Individual");
          Exp_Image img1 = getImage(ID1);
          Exp_Image img2 = getImage(ID2);
          try{
            ImagePairs.addElement(new Imagepair(p_ID,img1.getImage_ID(),img2.getImage_ID(),  Exp_ID, img1.getImage_Path(), img2.getImage_Path(), img1.getImage_Exp_ID(), img2.getImage_Exp_ID(), Same_Individual));
          }catch(Exception e){
            e.printStackTrace();
          }
       }while(res.next());
      }else{
        System.out.println("NO ELEMENTS FOUND:: Vector getImagePairs()");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
      return null;
    }catch(NullPointerException e){
      return null;
    }
    return ImagePairs;
  }

  private Exp_Image getImage(int Image_ID){
    Exp_Image img = null;

    Statement st;
    ResultSet res;
    String sql = "SELECT Image_ID, Image_path, Experiment_ID FROM Image WHERE Image_ID="+Image_ID;
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
          int img_ID = res.getInt("Image_ID");
          String path = res.getString("Image_path");
          int Exp_ID = res.getInt("Experiment_ID");
          try{
            img = new Exp_Image(img_ID,path,Exp_ID);
          }catch(Exception e){
            e.printStackTrace();
          }
      }else{
        System.out.println("NO ELEMENTS FOUND:: Image getImage(int Image_ID)");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
      return null;
    }catch(NullPointerException e){
      return null;
    }
    return img;
  }

  public Vector getExperiments()throws Exception{
    Statement st;
    ResultSet res;
    Vector v_exp = new Vector();
    String sql = "SELECT Experiment_ID, Experiment_name FROM Experiment_type ORDER BY Experiment_ID";
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
        do{
          int E_ID = res.getInt("Experiment_ID");
          String name = res.getString("Experiment_name");
          v_exp.addElement(E_ID+"#"+name);
        }while(res.next());
      }else{
        System.out.println("NO ELEMENTS FOUND:: Image getImage(int Image_ID)");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
        throw ex;
    }catch(NullPointerException e){
      throw new NullPointerException("No experiments were found in the database.\n"+e.getMessage());
    }
    return v_exp;
  }

  public Vector getParticipants(int exp_ID){
    Statement st;
    ResultSet res;
    Vector v_part = new Vector();
    String sql = "SELECT Experiment_ID, Participant_ID, Age, Gender, Highestdegree_ID, Job_with_ID_check FROM Participant WHERE Experiment_ID = "+exp_ID+" ORDER BY Participant_ID";
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
        do{
          int E_ID = res.getInt("Experiment_ID");
          int P_ID = res.getInt("Participant_ID");
          int A_ID = res.getInt("Age");
          int G_ID = res.getInt("Gender");
          int H_ID = res.getInt("Highestdegree_ID");
          int J_ID = res.getInt("Job_with_ID_check");
          v_part.addElement(E_ID+"#"+P_ID+"#"+A_ID+"#"+G_ID+"#"+H_ID+"#"+J_ID);
        }while(res.next());
      }else{
        System.out.println("NO ELEMENTS FOUND:: Image getImage(int Image_ID)");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
      return null;
    }catch(NullPointerException e){
      return null;
    }
    return v_part;
  }


  public int getActuallyAlike(int Imagepair_ID){
    Statement st;
    ResultSet res;
    String sql = "SELECT Imagepair_ID, Same_Individual FROM Imagepair WHERE Imagepair_ID="+Imagepair_ID;
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
          return res.getInt("Same_Individual");
      }else{
        System.out.println("NO ELEMENTS FOUND:: Image getImage(int Image_ID)");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
      return -1;
    }catch(NullPointerException e){
      return -1;
    }
    return -1;
  }

  public Vector getComparisons(int Exp_ID, int Part_ID){
    Statement st;
    ResultSet res;
    Vector v_part = new Vector();
    String sql = "SELECT Experiment_ID, Participant_ID, Imagepair_ID, Alike, Time_spent FROM Comparison WHERE Experiment_ID="+Exp_ID+" AND Participant_ID="+Part_ID+" ORDER BY Imagepair_ID";
    try{
      st = connection.createStatement();
      res = st.executeQuery(sql);
      if(res.next()){
        do{
          int E_ID = res.getInt("Experiment_ID");
          int P_ID = res.getInt("Participant_ID");
          int C_ID = res.getInt("Imagepair_ID");
          int A_ID = res.getInt("Alike");
          long T_ID = res.getLong("Time_spent");
          int Same_Individual = getActuallyAlike(C_ID);
//System.out.print("Alike: "+ A_ID);
          v_part.addElement(E_ID+"#"+P_ID+"#"+C_ID+"#"+A_ID+"#"+T_ID+"#"+Same_Individual);
        }while(res.next());
      }else{
        System.out.println("NO ELEMENTS FOUND:: Image getImage(int Image_ID)");
      }
      st.close();
    }catch(SQLException ex){
        ex.printStackTrace();
      return null;
    }catch(NullPointerException e){
      return null;
    }
    return v_part;
  }


}